#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include <winuser.h>

char	ProcessTitle[64] = "QWSV";
char	ProcessFound[64];
HANDLE	ProcessHandle;

DWORD	ExitCode;

int		Found = FALSE;
int		Error = FALSE;

int		LowPriority  = 8;
int		HighPriority = 0;


BOOL CALLBACK FindTitle( HWND hwnd, LPARAM lParam );

void main( int argc, char **argv )
{
	int i;

	printf( "Priority Controller (1.3) by Alan 'Strider' Kivlin\n" );
	printf( "Send bug reports/comments to: akivlin@saqnet.co.uk\n\n" );

	printf( "This program allows you to run a QuakeWorld server and a client on the same\nmachine with 0%% packetloss.\n\n" );

	if( argc > 1 )
	{
		for( i = 0; i < argc; i++ )
		{
			if( stricmp( argv[i], "-?" ) == 0 )
			{
				printf( "Usage: Priority <options>\n\n   [-low <low timer>]\n   [-high <high timer>]\n   [-title <window title of QWSV>]\n\n" );
				exit( 0 );
			}

			if( stricmp( argv[i], "-low" ) == 0 )
			{
				LowPriority = atoi( argv[i + 1] );

				if( LowPriority < 0 || LowPriority > 15 )
				{
					printf( "-low parameter must be in the range 0..15!\n\n" );
					exit( 0 );
				}
			}

			if( stricmp( argv[i], "-high" ) == 0 )
			{
				HighPriority = atoi( argv[i + 1] );

				if( HighPriority < 0 || HighPriority > 15 )
				{
					printf( "-high parameter must be in the range 0..15!\n\n" );
					exit( 0 );
				}
			}

			if( stricmp( argv[i], "-title" ) == 0 )
			{
				memset( ProcessTitle, '\0', 64 );
				strncpy( ProcessTitle, argv[i + 1], 64 );
				printf( "Title: %s\n\n", ProcessTitle );
			}
		}
	}

	printf( "Priority settings: \n   Low  = %d\n   High = %d\n\n", LowPriority, HighPriority );

	EnumWindows( FindTitle, 0 );

	if( ! Found || Error )
	{
		if( ! Error )
		{
			printf( "ERROR: Process (%s) isn't running?!\n", ProcessTitle );
		}

		exit( 0 );
	}

	printf( "Found: %s...\n\n", ProcessFound );

	printf( "Priority class control enabled.\n" );
	printf( "Press <ESC> to exit (will also disable priority class control).\n\n" );

	if( ! SetPriorityClass( GetCurrentProcess(), HIGH_PRIORITY_CLASS ) )
	{
		printf( "ERROR: Unable to set Priority Controller to HIGH priority class.\n" );
		exit( 0 );
	}

	while( TRUE )
	{
		if( kbhit() )
		{
			if( getch() == 27 )
			{
				break;
			}
		}

		GetExitCodeProcess( ProcessHandle, (LPDWORD)&ExitCode );

		if( ExitCode != STILL_ACTIVE )
		{
			printf( "Process (%s) has stopped running (%d).\n\n", ProcessFound, ExitCode );
			break;
		}

		if( ! SetPriorityClass( ProcessHandle, HIGH_PRIORITY_CLASS ) )
		{
			printf( "ERROR: Unable to set HIGH priority class.\n", ProcessFound );
			break;
		}

		Sleep( HighPriority );

		GetExitCodeProcess( ProcessHandle, (LPDWORD)&ExitCode );

		if( ExitCode != STILL_ACTIVE )
		{
			printf( "Process (%s) has stopped running (%d).\n\n", ProcessFound, ExitCode );
			break;
		}

		if( ! SetPriorityClass( ProcessHandle, NORMAL_PRIORITY_CLASS ) )
		{
			printf( "ERROR: Unable to set NORMAL priority class.\n" );
			break;
		}

		Sleep( LowPriority );
	}

	if( Found )
	{
		CloseHandle( ProcessHandle );
	}

	printf( "Exiting...\n" );

	exit( 0 );
}


BOOL CALLBACK FindTitle( HWND hwnd, LPARAM lParam )
{
	char	Title[64];
	DWORD	ThreadId;
	DWORD	ProcessId;

	GetWindowText( hwnd, (LPTSTR)Title, 64 );
	memcpy( ProcessFound, Title, 64 );

	if( strstr( strupr( Title ), strupr( ProcessTitle ) ) != NULL )
	{
		ThreadId = GetWindowThreadProcessId( hwnd, (LPDWORD)&ProcessId );

		if( ( ProcessHandle = OpenProcess( PROCESS_SET_INFORMATION + PROCESS_QUERY_INFORMATION, 0, ProcessId ) ) != NULL )
		{
			Found = TRUE;
		}
		else
		{
			Error = TRUE;
			printf( "ERROR: Unable to get process handle.\n" );
		}
	}

	return( ! Found && ! Error );
}
